/*
    5-11-07
    Nathan Seidle
    nathan@sparkfun.com
    Copyright Spark Fun Electronics 2007
    
    Software SPI control
    PIC 16F88
    
    Reading and writing to MLX90609 Digital/Analog Gyro
    
    This is a 11-bit, high precision digital (and analog) gyro
    Outputs temperature and rate
    
*/
#define Clock_8MHz
#define Baud_9600

#include "C:\Global\Code\C\16F88.h"  //Device dependent interrupt definitions

#pragma origin 4

#include "C:\Global\Code\Pics\Code\Delay.c"   //Standard delays
#include "C:\Global\Code\Pics\Code\stdio.c"   //Software based Basic Serial IO

void boot_up(void);
void mlx_test(void);
uns8 spi_comm(uns8 outgoing_byte);

void main(void)
{
    uns8 choice;
    
    boot_up();
        
    mlx_test();

    while(1);

}//End Main


void boot_up(void)
{
    //OSCCON = 0b.0111.0000; //Setup internal oscillator for 8MHz
    //while(OSCCON.2 == 0); //Wait for frequency to stabilize

    //Setup Ports
    ANSEL = 0b.0000.0000; //Turn off A/D

    TRISA = 0b.0000.0000;
    PORTA = 0b.0000.0000;

    TRISB = 0b.0001.0000;
    PORTB = 0b.0001.0000;   //0 = Output, 1 = Input RX on RB4

    //Setup the hardware UART module
    //=============================================================
    SPBRG = 51; //8MHz for 9600 inital communication baud rate
    //SPBRG = 129; //20MHz for 9600 inital communication baud rate

    TXSTA = 0b.0010.0100; //8-bit asych mode, high speed uart enabled
    RCSTA = 0b.1001.0000; //Serial port enable, 8-bit asych continous receive mode
    //=============================================================

}

//Read data from MLX90609 via SPI
void mlx_test(void)
{
                   
#define SCK PORTB.0 //SCLK This is clock input into MLX90609
#define SS   PORTB.1 //SS This is chip select input - when low, the MLX is selected
#define MISO PORTB.4 //MISO This is output from MLX into the PIC
#define MOSI PORTB.6 //MOSI This is input into the MLX from the PIC

    uns16 temp;
    uns8 data1, data2;

    //0 = Output, 1 = Input
    TRISA = 0b.0000.0000;  //Set all to outputs
    PORTA = 0b.0000.0000; //Set all outputs low
    
    //MISO = RB4
    TRISB = 0b.0001.0000;   //0 = Output, 1 = Input-RX on RB4
    PORTB = 0b.0001.0000;   //Set RB4 High

    while(1)
    {

        SS = 0; //Select MLX
        spi_comm(0b.1001.0100); // Read rate ADCC instruction
        SS = 1; //Unselext MLX
        
        delay_us(250); //Wait for MLX to take measurement
                
        SS = 0; //Select MLX
        spi_comm(0b.1000.0000); // Send SPI ADCR Instruction        
        data1 = spi_comm(0x00);
        data2 = spi_comm(0x00);
        SS = 1; //Unselect MLX
        
        if (data1.7 == 0 && data1.5 == 1)
        {
            temp.high8 = data1;
            temp.low8 = data2;
            temp >>= 1;
            temp &= 0b.0000.0111.1111.1111;
    
            printf("Rate=%d ", temp);
        }

        SS = 0; //Select MLX
        spi_comm(0b.1001.1100); // Read temperature ADCC instruction
        SS = 1; //Unselext MLX
        
        delay_us(250); //Wait for MLX to take measurement
                
        SS = 0; //Select MLX
        spi_comm(0b.1000.0000); // Send SPI ADCR Instruction        
        data1 = spi_comm(0x00);
        data2 = spi_comm(0x00);
        SS = 1; //Unselect MLX

        if (data1.7 == 0 && data1.5 == 1)
        {
            temp.high8 = data1;
            temp.low8 = data2;
            temp >>= 1;
            temp &= 0b.0000.0111.1111.1111;
    
            printf("Temp=%d\n", temp);
        }
    }
}

//Basic SPI send and receive
uns8 spi_comm(uns8 outgoing_byte)
{
    uns8 incoming_byte, x;

    for(x = 0 ; x < 8 ; x++)
    {
        SCK = 0; //Toggle the SPI clock

        MOSI = outgoing_byte.7; //Put bit on SPI data bus
        outgoing_byte <<= 1; //Rotate byte 1 to the left

        SCK = 1;

        incoming_byte <<= 1; //Rotate byte 1 to the left
        incoming_byte.0 = MISO; //Read bit on SPI data bus
    }

    return(incoming_byte);
}
